@@ -302,12 +302,10 @@ static Bool i915_texture_setup(PicturePtr picture, PixmapPtr pixmap, int unit)
 	ScrnInfoPtr scrn = xf86Screens[picture->pDrawable->pScreen->myNum];
 	intel_screen_private *intel = intel_get_screen_private(scrn);
 	uint32_t format, pitch, filter;
-	int w, h, i;
 	uint32_t wrap_mode, tiling_bits;
+	int i;
 
 	pitch = intel_pixmap_pitch(pixmap);
-	w = picture->pDrawable->width;
-	h = picture->pDrawable->height;
 	intel->scale_units[unit][0] = 1. / pixmap->drawable.width;
 	intel->scale_units[unit][1] = 1. / pixmap->drawable.height;
 
@@ -546,8 +544,8 @@ i915_emit_composite_primitive(PixmapPtr dest,
 {
 	ScrnInfoPtr scrn = xf86Screens[dest->drawable.pScreen->myNum];
 	intel_screen_private *intel = intel_get_screen_private(scrn);
-	Bool is_affine_src, is_affine_mask = TRUE;
-	int per_vertex, num_floats;
+	Bool is_affine_src = TRUE, is_affine_mask = TRUE;
+	int per_vertex;
 	int tex_unit = 0;
 	int src_unit = -1, mask_unit = -1;
 	float src_x[3], src_y[3], src_w[3], mask_x[3], mask_y[3], mask_w[3];
@@ -666,8 +664,6 @@ i915_emit_composite_primitive(PixmapPtr dest,
 		}
 	}
 
-	num_floats = 3 * per_vertex;
-
 	OUT_VERTEX(dstX + w);
 	OUT_VERTEX(dstY + h);
 	if (! intel->render_source_is_solid) {
@@ -1197,10 +1193,8 @@ i915_vertex_flush(intel_screen_private *intel)
 }
 
 void
-i915_batch_flush_notify(ScrnInfoPtr scrn)
+i915_batch_commit_notify(intel_screen_private *intel)
 {
-	intel_screen_private *intel = intel_get_screen_private(scrn);
-
 	intel->needs_render_state_emit = TRUE;
 	intel->render_current_dest = NULL;
 	intel->last_floats_per_vertex = 0;
